@@ -1,6 +1,6 @@
 use 5.008007;
 package Test::ModuleVersion;
-our $VERSION = '0.07';
+our $VERSION = '0.09';
 
 package
   Test::ModuleVersion::Object::Simple;
@@ -2573,14 +2573,16 @@ use strict;
 use warnings;
 use ExtUtils::Installed;
 use Carp 'croak';
+use Data::Dumper;
 
 sub has { __PACKAGE__->Test::ModuleVersion::Object::Simple::attr(@_) }
-has comment => '';
+has before => '';
 has distnames => sub { {} };
 has default_ignore => sub { ['Perl', 'Test::ModuleVersion'] };
 has ignore => sub { [] };
 has lib => sub { [] };
 has modules => sub { [] };
+has privates => sub { {} };
 
 sub detect {
   my $self = shift;
@@ -2595,44 +2597,43 @@ sub detect {
 }
 
 sub get_module_url {
-  my ($self, $module, $version) = @_;
+  my ($self, $module, $version, $opts) = @_;
   
+  $opts ||= {};
+  my $distnames = $opts->{distnames} || {};
+  my $privates = $opts->{privates} || {};
+
   # Module
   my $module_dist = $module;
-  my $distnames = $self->distnames;
   $module_dist = $distnames->{$module} if defined $distnames->{$module};
   $module_dist =~ s/::/-/g;
-  
-  # Get dounload URL using metaCPAN api
-  my $metacpan_api = 'http://api.metacpan.org/v0';
-  my $search = "release/_search?q=name:$module_dist-$version"
-    . "&fields=download_url,name";
-  my $http = Test::ModuleVersion::HTTP::Tiny->new;
-  my $res = $http->get("$metacpan_api/$search");
-  if ($res->{success}) {
-    my $release = Test::ModuleVersion::JSON::PP::decode_json $res->{content};
-    return $release->{hits}{hits}[0]{fields}{download_url};
+
+  if (my $url = $privates->{$module}) {
+    $url =~ s/%M/"$module_dist-$version"/e;
+    return $url;
+  }
+  else {
+    
+    # Get dounload URL using metaCPAN api
+    my $metacpan_api = 'http://api.metacpan.org/v0';
+    my $search = "release/_search?q=name:$module_dist-$version"
+      . "&fields=download_url,name";
+    my $http = Test::ModuleVersion::HTTP::Tiny->new;
+    my $res = $http->get("$metacpan_api/$search");
+    if ($res->{success}) {
+      my $release = Test::ModuleVersion::JSON::PP::decode_json $res->{content};
+      return $release->{hits}{hits}[0]{fields}{download_url};
+    }
   }
   
   return;
 }
 
 sub test_script {
-  my $self = shift;
+  my ($self, %opts) = @_;
   
   # Code
-  my $code;
-  
-  # Comment
-  my $comment = $self->comment;
-  $code .= <<"EOS" if $comment;
-=pod
-
-$comment
-
-=cut
-
-EOS
+  my $code = $self->before . "\n";
   
   # Reffer this module
   $code .= "# Created by Test::ModuleVersion $Test::ModuleVersion::VERSION\n";
@@ -2655,9 +2656,14 @@ EOS
 
 sub main {
   my $command = shift;
-  die qq/command "$command" is not found/
-    if defined $command && ($command ne 'list_fail' && $command ne 'list');
-
+  my $option = shift;
+  
+  die qq/command "$command" is unkonwn command/
+    if defined $command && $command ne 'list';
+  
+  die qq/list $option is unknown option/
+    if defined $option && $option ne '--fail';
+  
   if (defined $command) {
     my $builder = Test::More->builder;
     open my $out_fh, '>', undef;
@@ -2694,13 +2700,18 @@ EOS
   $code .= <<'EOS';
   # Print module URLs
   if (defined $command) {
+    my $distnames = <%%%%%% distnames %%%%%%>
+    ;
+    my $privates = <%%%%%% privates %%%%%%>
+    ;
     my $tm = Test::ModuleVersion->new;
-    my @ms = $command eq 'list_fail' ? @$failed
+    my @ms = $command eq 'list' && ($option || '') eq '--fail' ? @$failed
       : $command eq 'list' ? @$modules
-      : undef;
+      : [];
     for my $m (@ms) {
       my ($module, $version) = @$m;
-      my $url = $tm->get_module_url($module, $version);
+      my $url = $tm->get_module_url($module, $version,
+        {distnames => $distnames, privates => $privates});
       if (defined $url) { print "$url\n" }
       else { print STDERR "$module $version is unknown\n" }
     }  
@@ -2719,6 +2730,19 @@ EOS
   # Test count
   $code =~ s/<%%%%%% test_count %%%%%%>/$test_count/e;
   
+  # Distribution names
+  my $distnames_code = Data::Dumper->new([$self->distnames])->Terse(1)->Indent(2)->Dump;
+  $code =~ s/<%%%%%% distnames %%%%%%>/$distnames_code/e;
+
+  # Private repositories
+  my $privates_code = Data::Dumper->new([$self->privates])->Terse(1)->Indent(2)->Dump;
+  $code =~ s/<%%%%%% privates %%%%%%>/$privates_code/e;
+  
+  if (my $file = $opts{output}) {
+    open my $fh, '>', $file
+      or die qq/Can't open file "$file": $!/;
+    print $fh $code;
+  }
   return $code;
 }
 
@@ -2840,12 +2864,12 @@ It is very useful because you can know the module differnce.
 
 =head2 Get module URLs
 
-If test fail, you install the module manually, it is very hard work.
-you can get lacking module URLs by C<list_fail> command.
+If test fail, you usually install the module manually, it is very hard work.
+you can get module URLs by C<list> command.
 
-  $ perl module.t list_fail
+  $ perl module.t list
 
-The output is the following-like one.
+URLs of all module in test sciprt is printed.
 
   http://cpan.metacpan.org/authors/id/K/KI/KIMOTO/DBIx-Custom-0.2108.tar.gz
   ...
@@ -2854,38 +2878,51 @@ Internally, metaCPAN api is used to get module URL.
 if you have proxy server,
 you can set $ENV{http_proxy}.
 
-Installation using C<cpanm> is very easy.
+You can also get test failed module URL by C<--fail> option.
 
-  $ perl module.t list_fail | cpanm
+  $ perl module.t list --fail
 
-You can also print all modules in test by C<list> command.
+Module installation using L<cpanm> is very easy
+if cpanm is in current directory.
 
-  $ perl module.t list
+  $ perl t/module.t list --fail | perl cpanm -L extlib
+
+Modules is installed into C<extlib> directory.
 
 Have a fun to use L<Test::ModuleVersion>.
 
 =head1 ATTRIBUTES
 
-=head2 C<comment>
+=head2 C<before>
 
-  my $comment = $self->comment;
-  $tm = $tm->comment($comment);
+  my $code = $self->before;
+  $tm = $tm->before($code);
 
-Comment
+You can add some code before test script.
 
-You can embbed comment into test script.
+  $tm->before(<<'EOS');
+  use 5.008007;
+  
+  =pod
+  
+  You can create this script(t/module.t) by the following command.
 
-  $tm->comment(<<'EOS');
-  You can create this test script by the following command.
-
-    perl mvt.pl > t/module.t
+    perl mvt.pl
+  
+  =cut
+  
   EOS
 
 =head2 C<distnames>
 
   my $distnames = $self->distnames;
   $tm = $tm->distnames({
-    LWP => 'libwww-perl',
+    'LWP' => 'libwww-perl',
+    'IO::Compress::Base' => 'IO-Compress',
+    'Cwd' => 'PathTools',
+    'File::Spec' => 'PathTools',
+    'List::Util' => 'Scalar-List-Utils',
+    'Scalar::Util' => 'Scalar-List-Utils'
     ...
   });
 
@@ -2946,6 +2983,20 @@ Version number must be string like C<'0.1426'>, not C<0.1426>.
 
 If C<detect> method is executed, C<modules> attribute is set automatically.
 
+=head2 C<privates>
+
+  my $privates = $tm->privates;
+  $tm = $tm->privates({
+    'Some::Module' => 'http://localhost/~kimoto/%M.tar.gz'
+  });
+
+Private repogitory URL. If you use C<privates> attribute,
+you don't need upload module to C<CPAN>.
+You upload module to some place you can access by C<http> protocal.
+
+C<%M> is replaces module name and version number, like Some-Module-0.01.
+You also use this attribute with C<distnames> attribute.
+
 =head1 METHODS
 
 =head2 C<detect>
